#include <TinyGPS.h>
#include <SoftwareSerial.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>



 //constantes para controlar los LEDS
const int ledPinVerdeGPS = 2;
const int ledPinLecturaPH = 5;
const int ledPinRojoGPS = 6;

//constantes  para boton buscar posicion GPS
const int boton_GPS = 11;

//constantes  para modo calibración
const int boton_Calibracion = 10;

//Variables para control de botones
int boton_GPS_Estado=0;
int boton_Calibracion_Estado=0;

const int numPosCorrectas =5; //numero de posiciones GPS correctas para dar por válida la posición.
                              //esta acción mejora notablemente la posición obtenida


#define SensorPin 3     // pH meter salida análogica al Arduino
#define precisionPH 10  // número de muestras a tomar la media
#define I2C_ADDR 0x3F   // direccion I2C para controlar el LCD - previamente calculado con un scanner I2C


int valido=0;               //variable de control si el GPS es correcto
boolean modoAutonomo=false; //variable para trabajar sin RED GSM
int tiempoI2C=20;           //Tiempo de espera necesario para no colapsar el puerto I2C en milisegundos.



unsigned long int avgValue;        //valor de media de las mediciones
unsigned long tiempoEspera=600000; //tiempo de espera respuesta GPS
int reinicios=0;                   //reinicios del modulo SIM900
unsigned long actualizaciones=0;   //Lecturas y actualizaciones mandadas
int buf[10],temp;



LiquidCrystal_I2C lcd(I2C_ADDR,2, 1, 0, 4, 5, 6, 7);
SoftwareSerial SIM900(7, 8); // Configura el puerto serial para el SIM900. Para el Arduino MEGA utilizar pines 10 y 11
SoftwareSerial ss(4,3);
TinyGPS gps;
float latitud,longitud;



int respuesta;      //respuesta de cada comando
char aux_str[50];

float valorPH=0;   //Valor de pH tomado de la sonda analógica.

void setup()
{
      //posicion mandada por defecto en caso de no coger señal GPS
      latitud=0;
      longitud=0;

    
     lcd.begin (20,4);    // Inicializar el display con 20 caraceres 4 lineas
     lcd.setBacklightPin(3,POSITIVE);
     lcd.setBacklight(HIGH);
    
     lcd.home ();                   // Inicio LCD
     lcd.print("ARDUNOX PH");
     delay(tiempoI2C);
     lcd.setCursor ( 0, 1 );        // 2ª Linea
     lcd.print("INFO PH");
     delay(tiempoI2C);
     lcd.setCursor ( 0, 2 );        // 3ª Linea
     lcd.print("GPS POSITION");
     delay(tiempoI2C);
     lcd.setCursor ( 0, 3 );        // 4ª Linea
     lcd.print("By S.Pelayo");
    
     delay(1000);  //Tiempo espera para ver Letras inicio.
  
    //Configuración de Leds
    pinMode(ledPinRojoGPS, OUTPUT);
    pinMode(ledPinVerdeGPS, OUTPUT);
    pinMode(ledPinLecturaPH,OUTPUT); 
    //Estado inicial de Leds
    digitalWrite(ledPinRojoGPS, HIGH);
    digitalWrite(ledPinVerdeGPS, HIGH);
    digitalWrite(ledPinLecturaPH, LOW);
    
    
    // Inicializamos los pines de los botones como input:
  pinMode(boton_GPS, INPUT);
  pinMode(boton_Calibracion, INPUT);


  escribirLCD("Buscando Satelites",3);
  


    //Variable de control GPS inicializada a "no valido"
    valido=0;  //0=posicion NO valida  - 1=posicio valida
    
 getValidGPSPosition(tiempoEspera);  //Obtener posición con tiempo límite marcado.


   
  
  SIM900.begin(19200); //Configura velocidad del puerto serie para el SIM900

  delay(1000);
  escribirLCD("INICIO SIM OK",3);
      
  power_on();
  iniciarRed();
  delay(10000);
  iniciar();
  
}

void loop()
{    
  // Leemos los botones por si han sido pulsados
  boton_GPS_Estado = digitalRead(boton_GPS);
  boton_Calibracion_Estado = digitalRead(boton_Calibracion);
  
  if (boton_GPS_Estado==HIGH){  //Si se pulsa el boton GPS se machaca posicion GPS
    escribirLCD("REUBICANDO...",3);
    delay(3000);
    valido=0;
    getValidGPSPosition(tiempoEspera);  //Obtener posición con tiempo límite marcado.
  }
  
  if(boton_Calibracion_Estado==HIGH){ //Entramos en modo calibracion
    escribirLCD("CALIBRANDO",3);
    delay(3000);
    //Pulsar de nuevo para salir
    while((boton_Calibracion_Estado = digitalRead(boton_Calibracion))==LOW){
       valorPHLCD();
    }
    escribirLCD("FIN CALIBRADO",3);
  }
  
   valorPHLCD();   //Obtener valor pH y ponerlo en LCD
   valorGPSLCD();  //Poner situacion status GPS "OK-SIN DATOS"
   valorRyA();     // valor reinicio y actuyalizaciones en LCD

    delay(2000);
    PeticionHttp(); //Realiza petición al servidor y manda los datos.
  
}


int enviarAT(String ATcommand, char* resp_correcta, unsigned int tiempo)
{

  int x = 0;
  bool correcto = 0;
  char respuesta[100];
  unsigned long anterior;

  memset(respuesta, '\0', 100); // Inicializa el string
  delay(100);
  while ( SIM900.available() > 0) SIM900.read(); // Limpia el buffer de entrada
  SIM900.println(ATcommand); // Envia el comando AT
  x = 0;
  anterior = millis();
  // Espera una respuesta
  do {
    // si hay datos el buffer de entrada del UART lee y comprueba la respuesta
    if (SIM900.available() != 0)
    {
        respuesta[x] = SIM900.read();
        x++;
      // Comprueba si la respuesta es correcta
      if (strstr(respuesta, resp_correcta) != NULL)
      {
        correcto = 1;
      }
    }
  }
  // Espera hasta tener una respuesta
  while ((correcto == 0) && ((millis() - anterior) < tiempo));

  return correcto;
}

void power_on()
{
  int respuesta = 0;

  // Comprueba que el modulo SIM900 esta arrancado
  if (enviarAT("AT", "OK", 2000) == 0)
  {

    pinMode(9, OUTPUT);
    digitalWrite(9, HIGH);
    delay(1000);
    digitalWrite(9, LOW);
    delay(1000);

    // Espera la respuesta del modulo SIM900
    while (respuesta == 0) {
      // Envia un comando AT cada 2 segundos y espera la respuesta
      respuesta = enviarAT("AT", "OK", 2000);
      SIM900.println(respuesta);
    }
  }
}

void power_off()
{
  digitalWrite(9, HIGH);
  delay(1000);
  digitalWrite(9, LOW);
  delay(1000);
}

void reiniciar()
{

    
  power_off();
  delay (5000);
  power_on();
  reinicios=reinicios+1;
}
void iniciarRed(){
  int correcto=0;
  escribirLCD("INICIANDO RED GSM",3);

  correcto=enviarAT("AT+CPIN=\"xxxx\"", "OK", 1000); //Introducimos el PIN de la SIM
  if(!correcto) {
    correcto=enviarAT("AT", "OK", 1000);
    if(!correcto)
    {
      escribirLCD("D.Off:REINICIANDO",3);
      reiniciar();
    
    //Damos dos oportunidades para un inicio correcto (sino funcionaría sin conexión)
    correcto=enviarAT("AT+CPIN=\"xxxx\"", "OK", 1000);
    
  }
    
  }
  
 //conectando a la RED
  delay (5000);

  //Espera hasta estar conectado a la red movil
  while ( enviarAT("AT+CREG?", "+CREG: 0,1", 1000) == 0 )
  {
  }

 //LLegados a este punto estamos conectados
  
}

  
void iniciar()
{
  int correcto=0;
  SIM900.print("AT+CGDCONT=1,\"IP\",\"gprs-service.com\"");// Establece parametros PDP
  SIM900.print((char)13);
  delay(2000);

  escribirLCD("CONECTANDO A APN",3);
  correcto=enviarAT("AT+CSTT=\"gprs-service.com\",\"\",\"1234\"", "OK", 3000); //Definimos el APN, usuario y clave a utilizar
      
    
  if(correcto==0){
    escribirLCD("-SOCKET EN USO-",3);
    cerrarConexion();
    iniciar();
    return;
  }
  
  escribirLCD("SOLICITANDO CONEXION",3);
  enviarAT("AT+CIICR", "OK", 3000); //Activamos el perfil de datos inalámbrico
  enviarAT("AT+CIFSR", "", 3000); //Activamos el perfil de datos inalámbrico
  
}

void PeticionHttp()
{
  char petDispositivo[20];
  char petValor[20];

  int correcto=0;
  unsigned long tiempoPeticion;
tiempoPeticion=millis();
escribirLCD("PETICION PAGINA",3);
    
SIM900.print("AT+CIPSTART=\"TCP\",\"www.xxxx.es\",\"80\"");// Inicia conexion TCP o UDP
SIM900.print((char)13);

delay(5000);
SIM900.print("AT+CIPSEND");// Envias Datos TCP o UDP
SIM900.print((char)13);
delay(2000);

//Archivo donde se enviara el dato y el parametro a enviar
SIM900.print("GET /xxxx/inserta.php?");
SIM900.print("dispositivo=8");
SIM900.print("&valor=");
SIM900.print(valorPH);
SIM900.print("&latitud=");
SIM900.print(latitud,6);
SIM900.print("&longitud=");
SIM900.print(longitud,6);

SIM900.print(" HTTP/1.1");

SIM900.print((char)13);
SIM900.print((char)10);
SIM900.print("Host: www.xxxx.es");//
SIM900.print((char)13);
SIM900.print((char)10);
SIM900.print("Connection: Keep-Alive");
SIM900.print((char)13);
SIM900.print((char)10);
SIM900.print((char)13);
SIM900.print((char)10);
SIM900.print("\x1A"); // Finalizacion del comando CTRL+Z

correcto=enviarAT("AT+CIPCLOSE", "CLOSE OK", 3000);
if(correcto==0){

    escribirLCD("-FALLO CIERRE-",3);
    delay(500);
   

    
    cerrarConexion();
    reiniciar();
    iniciarRed();
    delay(10000);
    iniciar();

    return;
  }

  escribirLCD("PETICION FINALIZADA",3);
  delay(1000);

      escribirLCD("",3);
      lcd.print((millis()-tiempoPeticion)/1000 );
      delay(tiempoI2C);
      lcd.print(" sg.");
actualizaciones=actualizaciones+1;
delay(1000);



}

void cerrarConexion(void){
  //Cerramos y abrimos también la conexión serial  
    SIM900.end();
    delay(20);
    SIM900.begin(19200);
    
escribirLCD("CERRANDO",3);
    
SIM900.print("AT+CIPCLOSE");// Cierra la conexión TCP o UDP
SIM900.print((char)13);
delay(2000);
SIM900.print("AT+CIPSHUT"); //Cierra el contesto PDP de GPRS
SIM900.print((char)13);
        
}


void getValidGPSPosition(unsigned long tiempo){
 unsigned long  tiempoInicio;
    bool newData = false;
  unsigned long chars;
  unsigned short sentences, failed;
  int posiciones;
  posiciones=0;
  ss.begin(9600);  //Arrancamos el GPS con velocidad de comunicacion de 9600
  //La siguiente función determina la posición GPS inicial; 
  //al tratarse de un equipo de movilidad reducida una vez 
  //determinada la posición no la refrescamos a no ser que el cliente quiera.
  
tiempoInicio=millis();
while((valido==0 || posiciones<numPosCorrectas) && (millis()-tiempoInicio)<tiempo) //condicion para terminar
{
  // Analizamos datos GPS y palabras clave por 1 sg.
  for (unsigned long start = millis(); millis() - start < 1000;)
  {
    while (ss.available())
    {
      char c = ss.read();
    if (gps.encode(c)) // Nuevo dato encontrado con validez
        newData = true;
    }
  }

  if (newData)
  {
    posiciones=posiciones+1;
    //Mostramos en pantalla el estado y la el numero de posiciones correctas encontradas
     escribirLCD("POS. ENCONTRADA ",3);
     lcd.print(posiciones);
     delay(tiempoI2C);
     lcd.print("/");
     delay(tiempoI2C);
     lcd.print(numPosCorrectas);
     delay(tiempoI2C);
     
    valido=1;
    float flat, flon;
    unsigned long age;
    gps.f_get_position(&flat, &flon, &age);
    latitud=flat;
    longitud=flon;

  }
  gps.stats(&chars, &sentences, &failed);
  
  //No pausamos la lectura de pH mientras busca posición
  valorPHLCD();
  
}
//Ponemos el estado de validez en los leds de información (Rojo=SIN GPS // verde=GPS OK )
  if(valido==0) 
    {
      digitalWrite(ledPinRojoGPS, HIGH);digitalWrite(ledPinVerdeGPS, LOW);
      escribirLCD("SIN GPS",3);
    }
     else 
    {
      digitalWrite(ledPinRojoGPS, LOW);digitalWrite(ledPinVerdeGPS, HIGH);
      escribirLCD("GPS OK",3);
    }
    
 ss.end(); 
} 


/*
Formato de la linea del modulo GPS recibida que se usa.
Recommended minimum specific GPS/Transit data
http://aprs.gids.nl/nmea/#rmc


$GPRMC,044235.000,A,4322.0289,N,00824.5210,W,0.39,65.46,020615,,,A*44

- 044235.000 representa la hora GMT (04:42:35)
- “A” es la indicación de que el dato de posición está fijado y es correcto. “V” sería no válido
- 4322.0289 representa la longitud (43º 22.0289´)
- N representa el Norte
- 00824.5210 representa la latitud (8º 24.5210´)
- W representa el Oeste
- 0.39 representa la velocidad en nudos
- 65.46 representa la orientación en grados
- 020615 representa la fecha (2 de Junio del 2015)
*/

float getpH(){
  for(int i=0;i<precisionPH;i++)       //toma de muestras (segun la constante definida precisionPH)
  { 
    buf[i]=analogRead(SensorPin);
    delay(10);
  }
  for(int i=0;i<precisionPH-1;i++)        //Ordenar muestras
  {
    for(int j=i+1;j<precisionPH;j++)
    {
      if(buf[i]>buf[j])
      {
        temp=buf[i];
        buf[i]=buf[j];
        buf[j]=temp;
      }
    }
  }
  avgValue=0;
  for(int i=2;i<precisionPH-2;i++)                      //Se hace la media de las muestras intermedias
    avgValue+=buf[i];
  float phValue=(float)avgValue*5.0/1024/6; //se convierten las mediciones a milivoltios
  phValue=3.5*phValue;                      //se convierten los milivoltios en valor de pH

  digitalWrite(ledPinLecturaPH, HIGH);       
  delay(800);
  digitalWrite(ledPinLecturaPH, LOW);
  return phValue;
}

void escribirLCD(String msg,int linea){
  lcd.setCursor(0,linea);
  delay(tiempoI2C);
  lcd.print("                    ");
  delay(tiempoI2C);
  lcd.setCursor(0,linea);
  delay(tiempoI2C);
  lcd.print(msg);
  delay(tiempoI2C);
}

void valorPHLCD(){
    valorPH=getpH();
    lcd.setCursor ( 0, 0 );
    delay(tiempoI2C);
    lcd.print("VALOR:");
    delay(tiempoI2C);
    lcd.print(valorPH);   
}

void valorGPSLCD(){
   escribirLCD("GPS -> ",1);
   if(valido)
     lcd.print("OK");
   else
      lcd.print("SIN DATOS");  
 delay(tiempoI2C);
}

void valorRyA(){
 escribirLCD("R:",2); 
 lcd.print(reinicios);
 delay(tiempoI2C);
 lcd.print(" A:");
 delay(tiempoI2C);
 lcd.print(actualizaciones); 
}


